<?php
namespace Teknisa\Model\Geral\Repositories;

use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use Teknisa\Model\Geral\Entities\Estruturam;

class Estruturah extends \Doctrine\ORM\EntityRepository {
    
    public function retornaEstruturaHistorico($nrorg, $nrestruturam, $dtmescompetenc){

	$params = array('nrestruturam' => $nrestruturam, 'nrorg'=>$nrorg, 'dtmescompetenc' => $dtmescompetenc);
    $estruturah =  '\Teknisa\Model\Geral\Entities\Estruturah';

$sql = <<<SQL
SELECT e FROM $estruturah e
WHERE e.nrorg = :nrorg
    AND e.nrestruturam = :nrestruturam    
    AND e.dtmescompetenc = 
    (SELECT MAX(ee.dtmescompetenc) FROM $estruturah ee
    WHERE ee.nrestruturam = e.nrestruturam
    AND ee.nrorg = e.nrorg
    AND ee.dtmescompetenc <= :dtmescompetenc
    )
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result[0];
    }
    
    public function retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $nrinscricao, $nrtipoestrutura, $dtmescompetenc){

	$params = array('nrinscricao' => $nrinscricao, 
	                'nrorg'=>$nrorg, 
	                'nrtipoestrutura' => $nrtipoestrutura,
	                'dtmescompetenc' => $dtmescompetenc);
	                
    $estruturah =  '\Teknisa\Model\Geral\Entities\Estruturah';
    $estruturam =  '\Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT em,eh 
FROM $estruturam em
LEFT JOIN $estruturah eh WITH eh.nrorg = em.nrorg
               AND eh.nrestruturam = em.nrestruturam
               AND eh.dtmescompetenc =  (SELECT MAX(ee.dtmescompetenc) FROM $estruturah ee
                                        WHERE ee.nrestruturam = eh.nrestruturam
                                        AND ee.nrorg = eh.nrorg
                                        AND ee.dtmescompetenc <= :dtmescompetenc)
WHERE em.nrorg = :nrorg
    AND ((eh.cdcnpjestrut = :nrinscricao AND eh.cdceiestrut IS NULL AND eh.cdcaepfestrut  IS NULL )
          OR eh.cdceiestrut = :nrinscricao
          OR eh.cdcaepfestrut = :nrinscricao)
    AND em.nrtipoestrutura = :nrtipoestrutura
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    public function retornaEstruturaMatrizPorRaizCnpj($nrorg, $nrinscricao, $nrtipoestrutura, $dtmescompetenc){
        
        $dtultdiacompet = DateUtil::getUltimoDiaDoMes($dtmescompetenc)->format('d/m/Y');

    	$params = array('nrinscricao' => $nrinscricao, 
    	                'nrorg'=>$nrorg, 
    	                'nrtipoestrutura' => $nrtipoestrutura,
    	                'dtmescompetenc' => $dtmescompetenc,
                        'dtultdiacompet' => $dtultdiacompet
                    );
    	                
        $estruturah =  '\Teknisa\Model\Geral\Entities\Estruturah';
        $estruturam =  '\Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT em,eh 
FROM $estruturam em
LEFT JOIN $estruturah eh WITH eh.nrorg = em.nrorg
               AND eh.nrestruturam = em.nrestruturam
               AND eh.dtmescompetenc =  (SELECT MAX(ee.dtmescompetenc) FROM $estruturah ee
                                        WHERE ee.nrestruturam = eh.nrestruturam
                                        AND ee.nrorg = eh.nrorg
                                        AND ee.dtmescompetenc <= :dtmescompetenc)
WHERE em.nrorg = :nrorg
    AND SUBSTRING(eh.cdcnpjestrut,1,8) = SUBSTRING(:nrinscricao,1,8)
    AND SUBSTRING(eh.cdcnpjestrut,9,4) = '0001'
    AND em.nrtipoestrutura = :nrtipoestrutura
    AND em.dtinivigencia <= :dtultdiacompet AND (em.dtfimvigencia IS NULL OR em.dtfimvigencia >= :dtmescompetenc)
ORDER BY em.nrestruturam
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    public function retornaEstruturasPorRaizCnpj($nrorg, $nrinscricao, $nrtipoestrutura, $dtmescompetenc, $idativo='S'){
        
        $dtultdiacompet = DateUtil::getUltimoDiaDoMes($dtmescompetenc)->format('d/m/Y');

    	$params = array(
    	    'nrinscricao' => $nrinscricao, 
            'nrorg'=>$nrorg, 
            'nrtipoestrutura' => $nrtipoestrutura,
            'dtmescompetenc' => $dtmescompetenc,
            'dtultdiacompet' => $dtultdiacompet,
            'idativo' => is_null($idativo) ? 'S' : $idativo,
        );
    	                
        $estruturah =  'Teknisa\Model\Geral\Entities\Estruturah';
        $estruturam =  'Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT em,eh 
FROM $estruturam em
LEFT JOIN $estruturah eh WITH eh.nrorg = em.nrorg
               AND eh.nrestruturam = em.nrestruturam
               AND eh.dtmescompetenc =  (SELECT MAX(ee.dtmescompetenc) FROM $estruturah ee
                                        WHERE ee.nrestruturam = eh.nrestruturam
                                        AND ee.nrorg = eh.nrorg
                                        AND ee.dtmescompetenc <= :dtmescompetenc)
WHERE em.nrorg = :nrorg
  AND SUBSTRING(eh.cdcnpjestrut,1,8) = SUBSTRING(:nrinscricao,1,8)
  AND em.nrtipoestrutura = :nrtipoestrutura
  AND ((:idativo = 'S' AND em.dtinivigencia <= :dtultdiacompet AND (em.dtfimvigencia IS NULL OR em.dtfimvigencia >= :dtmescompetenc)) OR :idativo <> 'S')
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
        $estruturas = array();
        
        foreach($result as $obj) {
            if (get_class($obj) == $estruturam) {
                $estruturas[] = $obj;
            } else if (get_class($obj) == $estruturah) {
                $estruturas[count($estruturas)-1]->historico = $obj;
            }
        }
        
        return $estruturas;
    }
    
    public function retornaEstruturaMatrizPorFilial($nrorg, $nrinscricao, $nrtipoestrutura, $dtmescompetenc){
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();

    	$params = array('P_NRINSCR' => $nrinscricao, 
    	                'P_NRORG'=>$nrorg, 
    	                'P_NRTIPOESTRUTLEGAL' => $nrtipoestrutura,
    	                'P_DTMESCOMPETENC' => $dtmescompetenc);

$sql = <<<SQL
SELECT EM.*,
       EH.*
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(ESTRUTURAH.DTMESCOMPETENC) FROM ESTRUTURAH 
                                              WHERE ESTRUTURAH.NRORG = EH.NRORG
                                                AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
   AND EM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
   AND (EM.DTFIMVIGENCIA IS NULL OR EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
   AND EH.CDCEIESTRUT IS NULL
   AND EH.CDCAEPFESTRUT  IS NULL
   AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = :P_NRINSCR
   AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),9,4) = '0001'
SQL;
        $result = $db->fetchAll($sql,$params);
        return $result;
    }
    
    public function getFeriados($dtinicio, $dtfinal, $nrestruturam, $dtmescompetencia, $nrorg)
    {
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $estruturah =  Repositories::ESTRUTURAH;
        $ocorcalendario = Repositories::OCORCALENDARIO;
        
        $params = [
                    'p_nrorg'=>$nrorg, 
                    'p_nrestruturam' => $nrestruturam, 
                    'p_dtmescompetenc' => $dtmescompetenc,
                  ];     

        $sql = <<<SQL
SELECT DISTINCT OCORCALENDARIO.DTCALENDARIO
FROM ESTRUTURAH
INNER JOIN OCORCALENDARIO
    ON OCORCALENDARIO.NRCALENDARIO = ESTRUTURAH.NRCALENDARIO
    AND OCORCALENDARIO.NRORG = ESTRUTURAH.NRORG
WHERE ESTRUTURAH.NRORG = $nrorg
    AND OCORCALENDARIO.DTCALENDARIO BETWEEN '$dtinicio' AND '$dtfinal'
    AND ESTRUTURAH.NRESTRUTURAM = $nrestruturam
    AND ESTRUTURAH.DTMESCOMPETENC = ( SELECT MAX(MAX_ESTRUTURAH.DTMESCOMPETENC)
                                    FROM ESTRUTURAH MAX_ESTRUTURAH
                                    WHERE MAX_ESTRUTURAH.DTMESCOMPETENC <= '$dtmescompetencia'
                                        AND MAX_ESTRUTURAH.NRORG = ESTRUTURAH.NRORG
                                        AND MAX_ESTRUTURAH.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM)
SQL;
        $result = $db->fetchAll($sql,$params);
        
        if($result)
            return $result;
        else
            return array();
    }  
    
    public function getEstruturaPorNrInsc($nrorg, $dtmescompetenc, $prefixocnpj, $tpinsc){
        
        $estruturah =  '\Teknisa\Model\Geral\Entities\Estruturah';
        
        $sql = <<<SQL
SELECT eh 
  FROM $estruturah eh
 WHERE eh.nrorg = :p_nrorg
   AND ((substring(eh.cdcnpjestrut, 0, 12) = CONCAT(:p_prefixocnpj, '0001') AND :p_tpinsc = 1)
        OR (substring(eh.cdcnpjestrut, 0, 8) = :p_prefixocnpj AND :p_tpinsc = 2))
   AND eh.dtmescompetenc = (SELECT max(ehh.dtmescompetenc)
                              FROM $estruturah ehh
                             WHERE ehh.nrorg = eh.nrorg
                               AND ehh.nrestruturam = eh.nrestruturam
                               AND ehh.dtmescompetenc <= :p_dtmescompetenc)
SQL;
        
        $params = array(
            'p_prefixocnpj' => $prefixocnpj,
            'p_tpinsc' => $tpinsc,
            'p_dtmescompetenc' => $dtmescompetenc,
            'p_nrorg' => $nrorg
        );

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        $result = ($result)?$result[0]:false;
        return $result;
    }
    
    public function retornaSindicatosComHierarquia($nrestruturamsup, $nrorg, $nrorgpadrao){
        
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
$sql = <<<SQL
        SELECT
            TPH.NRTIPOESTRUTSUP,
            TPH.NRTIPOESTRUTREL,
            TPH.NRORG,
            HO.NRESTRUTURAMREL,
            ESTRUT_SUP.NRESTRUTURAM    AS NRESTRUTLEGAL,
            ESTRUT_SUP.NMESTRUTURAM    AS NMRAZSOCESTRUT_LEGAL,
            ESTRUT_SUP.NRTIPOESTRUTURA AS NRTIPOESTRUTURA_LEGAL
        FROM
                 TIPOHIERARQORG TPH
            JOIN HIERARQUIAORG HO ON HO.NRTIPOHIERARQORG = TPH.NRTIPOHIERARQORG
                                     AND ( HO.NRORG = :P_NRORG
                                           OR HO.NRORG = :P_NRORG_PADRAO )
            JOIN GPE_PARAMORG PO ON TPH.NRTIPOESTRUTSUP = PO.NRTPESTRUTLEGAL 
                                    AND  (PO.NRORG = :P_NRORG
                                           OR PO.NRORG = :P_NRORG_PADRAO )
            JOIN ESTRUTURAM  ESTRUT_SUP ON ESTRUT_SUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                                          AND ESTRUT_SUP.NRORG = :P_NRORG
                                          AND ESTRUT_SUP.NRTIPOESTRUTURA = PO.NRTPESTRUTLEGAL
        WHERE
            (TPH.NRORG = :P_NRORG OR TPH.NRORG = :P_NRORG_PADRAO) AND
            TPH.NRTIPOESTRUTREL = 10 AND
            ESTRUT_SUP.NRESTRUTURAM = :P_NRESTRUTLEGAL
SQL;
        
        $params = [
                    "P_NRESTRUTLEGAL" => $nrestruturamsup,
                    "P_NRORG" => $nrorg,
                    "P_NRORG_PADRAO" => $nrorgpadrao,
                  ];
    
        $result = $db->fetchAll($sql,$params);
        
        if($result)
            return $result;
        else
            return array();         
        
    }

    public function retornaEstruturaMatrizPorEstruturaLegal($nrorg, $nrestrutlegal, $nrtipoestrutura, $dtmescompetenc)
    {   
        $dtultdiacompet = DateUtil::getUltimoDiaDoMes($dtmescompetenc)->format('d/m/Y');

    	$params = array('nrestrutlegal' => $nrestrutlegal, 
    	                'nrorg'=>$nrorg, 
    	                'nrtipoestrutura' => $nrtipoestrutura,
    	                'dtmescompetenc' => $dtmescompetenc,
                        'dtultdiacompet' => $dtultdiacompet
                    );
    	                
        $estruturah =  '\Teknisa\Model\Geral\Entities\Estruturah';
        $estruturam =  '\Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT em,eh 
FROM $estruturam em
LEFT JOIN $estruturah eh WITH eh.nrorg = em.nrorg
               AND eh.nrestruturam = em.nrestruturam
               AND eh.dtmescompetenc =  (SELECT MAX(ee.dtmescompetenc) FROM $estruturah ee
                                        WHERE ee.nrestruturam = eh.nrestruturam
                                        AND ee.nrorg = eh.nrorg
                                        AND ee.dtmescompetenc <= :dtmescompetenc)
LEFT JOIN $estruturah ehm WITH ehm.nrorg = eh.nrorg
               AND ehm.nrestruturam = :nrestrutlegal
               AND ehm.dtmescompetenc =  (SELECT MAX(ehm.dtmescompetenc) FROM $estruturah eem
                                        WHERE eem.nrestruturam = ehm.nrestruturam
                                        AND eem.nrorg = ehm.nrorg
                                        AND eem.dtmescompetenc <= :dtmescompetenc)
WHERE em.nrorg = :nrorg
    AND SUBSTRING(eh.cdcnpjestrut,1,8) = SUBSTRING(ehm.cdcnpjestrut,1,8)
    AND SUBSTRING(eh.cdcnpjestrut,9,4) = '0001'
    AND em.nrtipoestrutura = :nrtipoestrutura
    AND em.dtinivigencia <= :dtultdiacompet AND (em.dtfimvigencia IS NULL OR em.dtfimvigencia >= :dtmescompetenc)
ORDER BY em.nrestruturam
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }

    
}